###########################################################
## This is an R file that forms part of the project
## "Equation-by-Equation Estimation of Multivariate
## Periodic Electricity Price Volatility" by lvaro
## Escribano and Genaro Sucarrat.
##
## This program loads data and undertakes electricity
## price analysis
##
## First created 27 May 2016
## Last modified 15 January 2018
##
## Contents:
##
## 1 INITIATE
## 2 LOAD, TRANSFORM AND PREPARE NORDPOOL DATA
## 3 CONDITIONAL MEAN MODELS
## 4 LOG-GARCH MODELS OF CONDITIONAL VARIANCES
## 5 cDCC MODEL
## 6 SUGGESTION BY REVIEWER 2
##
###########################################################


##=========================================================
##1 INITIATE
##=========================================================

##remove everything in environment:
rm(list=ls())

##set working directory:
setwd("C:/Users/sucarrat/Documents/projects/escribano01/analysis/20160704/")
#setwd("C:/tmp/escribano01/analysis/20160327/")

##load lgarchmc code:
source("C:/Users/sucarrat/Documents/R/gs/lgarch/prototype/lgarch-multiplicative-component-source.R")
## NOTE: the code uses C-code that is compiled with the inline-package.
## In other words, Windows users must install Rtools for inline to work.

##load libraries
library(lgarch)
library(gets) #version 0.13 or later is required
library(xtable)

##=========================================================
##2 LOAD, TRANSFORM AND PREPARE NORDPOOL DATA
##=========================================================

mData1 <- read.csv("./data/nordpoolhourlyspot2010.csv")
mData2 <- read.csv("./data/nordpoolhourlyspot2011.csv")
mData3 <- read.csv("./data/nordpoolhourlyspot2012.csv")
mData4 <- read.csv("./data/nordpoolhourlyspot2013.csv")
mData5 <- read.csv("./data/nordpoolhourlyspot2014.csv")

##extract SYS, StartHour and Date:
price24hour <- c(as.numeric(mData1[,"SYS"]),
  as.numeric(mData2[,"SYS"]),
  as.numeric(mData3[,"SYS"]),
  as.numeric(mData4[,"SYS"]),
  as.numeric(mData5[,"SYS"]))
starthour <- c(as.numeric(mData1[,"StartHour"]), as.numeric(mData2[,"StartHour"]),
  as.numeric(mData3[,"StartHour"]), as.numeric(mData4[,"StartHour"]),
  as.numeric(mData5[,"StartHour"]))
date24hour <- c(as.character(mData1[,"Date"]), as.character(mData2[,"Date"]),
  as.character(mData3[,"Date"]), as.character(mData4[,"Date"]),
  as.character(mData5[,"Date"]))

##create matrix w/hourly system prices:
mHourlyPrices <- NULL
for(i in 0:23){
  vWhereis <- which(starthour==i)
  mHourlyPrices <- cbind(mHourlyPrices, price24hour[vWhereis])
}
colnames(mHourlyPrices) <- paste("hour", 0:23, sep="")
mHourlyPrices <- zoo(mHourlyPrices,
  order.by=as.Date(date24hour[vWhereis], "%d-%m-%Y"))

##adjust the spurious zeros at m=3:
whereZeros <- which(mHourlyPrices[,3]==0)
for(i in 1:length(whereZeros)){
  mHourlyPrices[ whereZeros[i] ,3] <- 0.5*coredata(mHourlyPrices[ c(whereZeros[i]-1) ,3]) + 0.5*coredata(mHourlyPrices[ c(whereZeros[i]+1) ,3])
}

##Figure 1: Hourly prices
##-----------------------
plot(mHourlyPrices, main="", xlab="", ylab=paste(1:24,sep=""),
  col="blue", lwd=1, ylim=c(0,227))

##create matrix w/hourly log-returns:
mHourlyReturns <- NULL
for(i in 1:24){
#  tmp <- coredata(mHourlyPrices[-1,i])/coredata(mHourlyPrices[-NROW(mHourlyPrices),i]) - 1
  mHourlyReturns <- cbind(mHourlyReturns,
    coredata(gdiff(log(mHourlyPrices[,i]))) )
}
colnames(mHourlyReturns) <- colnames(mHourlyPrices)
mHourlyReturns <- zoo(mHourlyReturns,
  order.by=index(mHourlyPrices))
head(mHourlyReturns)

##Figure 2: Hourly returns
##------------------------
plot(mHourlyReturns, main="", xlab="", ylab=paste(1:24,sep=""),
  col="blue", lwd=1)

##day-of-the-week dummies:
vDay <- weekdays(index(mHourlyReturns))
dMon <- zoo(as.numeric(vDay=="Monday"), order.by=index(mHourlyReturns))
dTue <- zoo(as.numeric(vDay=="Tuesday"), order.by=index(mHourlyReturns))
dWed <- zoo(as.numeric(vDay=="Wednesday"), order.by=index(mHourlyReturns))
dThu <- zoo(as.numeric(vDay=="Thursday"), order.by=index(mHourlyReturns))
dFri <- zoo(as.numeric(vDay=="Friday"), order.by=index(mHourlyReturns))
dSat <- zoo(as.numeric(vDay=="Saturday"), order.by=index(mHourlyReturns))
dSun <- zoo(as.numeric(vDay=="Sunday"), order.by=index(mHourlyReturns))
mDay <- cbind(dMon, dTue, dWed, dThu, dFri, dSat, dSun)


##=========================================================
##3 CONDITIONAL MEAN MODELS
##=========================================================

mMeanMod <- matrix(NA,24,4)
colnames(mMeanMod) <- c("R2", "tba", "tba", "tba")
mMeanModResids <- matrix(NA, 1593, 24)
colnames(mMeanModResids) <- paste("resids", 1:24, sep="")

for(i in 1:24){

  meanMod <- arx(mHourlyReturns[,i], mc=1, ar=c(1:7),
    mxreg=mDay[,-1], vcov.type="w", plot=FALSE)

  ##compute R-squared:
  tmp <- cbind(window(mHourlyReturns[,i],
    start=index(meanMod$residuals)[1]), meanMod$residuals)
  TSS <- sum( (tmp[,1] - mean(tmp[,1]))^2 )
  RSS <- sum( (tmp[,2] - mean(tmp[,2]))^2 )
  Rsq <- 1 - RSS/TSS
  mMeanMod[i,1] <- Rsq

  ##store residuals:
  mMeanModResids[,i] <- meanMod$residuals
}
mMeanMod
mMeanModResids <- zoo(mMeanModResids,
  order.by=index(meanMod$residuals))
head(mMeanModResids)


##=========================================================
##4 LOG-GARCH MODELS OF CONDITIONAL VARIANCES
##=========================================================

##matrix of extended leverage terms:
mAsymExt <- matrix(NA,NROW(mMeanModResids),NCOL(mMeanModResids))
for(i in 1:NCOL(mAsymExt)){
  mAsymExt[,i] <- coredata(as.numeric(mMeanModResids[,i] < 0))
}
colnames(mAsymExt) <- paste("asymext", 1:24, sep="")
mAsymExt <- zoo(mAsymExt, order.by=index(mMeanModResids))
head(mAsymExt)

##matrix of mean-corrected log-ARCH terms
##(the adjacency terms for models (d) and (e)):
##(note: this part can be moved to (b) in loop below)
mLogARCH <- log(mMeanModResids^2)
for(i in 1:24){
  varModDet <- arx(mMeanModResids[,i], mc=TRUE, mxreg=mDay[,-1])
  mLogARCH[,i] <- residuals(varModDet)
}

##matrix of estimation results:
mVarModels <- matrix(NA, 24*5, 10)
colnames(mVarModels) <- c("hour", "R2", "log-GARCH",
  "ARCH", "GARCH", "ASYM", "LogL", "k", "BIC", "T")
indx <- seq(1,24*5,by=5) #auxiliary index
mVarModels[indx, "hour"] <- 1:24
mVarModels[indx, "R2"] <- mMeanMod[,"R2"]

##matrix of standardised residuals:
mResidStd <- NULL
mResidStdNames <- NULL

##estimate models (a) to (e) loop:
t1 <- date() #start time
for(i in 1:24){

  ##spec type:
  mVarModels[indx[i]:c(indx[i]+4),"log-GARCH"] <- 1:5

  ##(a) log-garch(1,1):
  varMod <- lgarchmc(mMeanModResids[,i], arch=1, garch=1)
  mVarModels[indx[i],"ARCH"] <- varMod$par["arch1"]
  mVarModels[indx[i],"GARCH"] <- varMod$par["garch1"]
  mVarModels[indx[i],"LogL"] <- as.numeric(logLik(varMod))
  mVarModels[indx[i],"k"] <- length(varMod$par)-1
  mVarModels[indx[i],"T"] <- length(varMod$resids.std)
  mVarModels[indx[i],"BIC"] <- info.criterion(mVarModels[indx[i],"LogL"],
    n=mVarModels[indx[i],"T"],
    k=mVarModels[indx[i],"k"])$value
  if(i==19){ #when (a) provides best fit (see below)
    mResidStd <- cbind(mResidStd, coredata(residuals(varMod))[-1] )
    mResidStdNames <- c(mResidStdNames, "1")
  }

  ##(b) log-garch(0,0) w/periodicity:
  varMod <- arx(mMeanModResids[,i], vxreg=mDay[,-1], plot=FALSE)
  mVarModels[indx[i]+1,"LogL"] <- as.numeric(logLik(varMod))
  mVarModels[indx[i]+1,"k"] <- length(coef(varMod))-1
  mVarModels[indx[i]+1,"T"] <- length(varMod$std.residuals)
  mVarModels[indx[i]+1,"BIC"] <- info.criterion(mVarModels[indx[i]+1,"LogL"],
    n=mVarModels[indx[i]+1,"T"],
    k=mVarModels[indx[i]+1,"k"])$value
  #(b) never provides the best fit (see below)

  ##(c) log-garch(1,1) w/periodicity:
  varMod <- lgarchmc(mMeanModResids[,i], arch=1, garch=1,
    xregdet=mDay[,-1])
  mVarModels[indx[i]+2,"ARCH"] <- varMod$par["arch1"]
  mVarModels[indx[i]+2,"GARCH"] <- varMod$par["garch1"]
  mVarModels[indx[i]+2,"LogL"] <- as.numeric(logLik(varMod))
  mVarModels[indx[i]+2,"k"] <- length(varMod$par)-1
  mVarModels[indx[i]+2,"T"] <- length(varMod$resids.std)
  mVarModels[indx[i]+2,"BIC"] <- info.criterion(mVarModels[indx[i]+2,"LogL"],
    n=mVarModels[indx[i]+2,"T"],
    k=mVarModels[indx[i]+2,"k"])$value
  if(i==16){ #when (c) provides best fit (see below)
    mResidStd <- cbind(mResidStd, coredata(residuals(varMod))[-1] )
    mResidStdNames <- c(mResidStdNames, "3")
  }

  ##(d) log-garch(1,1) w/periodicity and feedback:
  if(i==1){
    mARCH <- cbind(glag(mLogARCH[,2]), glag(mLogARCH[,3]))
  }
  if(i %in% 2:23){
    mARCH <- cbind(glag(mLogARCH[,c(i-1)]), glag(mLogARCH[,c(i+1)]))
  }
  if(i==24){
    mARCH <- cbind(glag(mLogARCH[,22]), glag(mLogARCH[,23]))
  }
  varMod <- lgarchmc(mMeanModResids[-1,i], arch=1, garch=1,
    xregdet=mDay[,-1], xreg=mARCH)
  mVarModels[indx[i]+3,"ARCH"] <- varMod$par["arch1"]
  mVarModels[indx[i]+3,"GARCH"] <- varMod$par["garch1"]
  mVarModels[indx[i]+3,"LogL"] <- as.numeric(logLik(varMod))
  mVarModels[indx[i]+3,"k"] <- length(varMod$par)-1
  mVarModels[indx[i]+3,"T"] <- length(varMod$resids.std)
  mVarModels[indx[i]+3,"BIC"] <- info.criterion(mVarModels[indx[i]+3,"LogL"],
    n=mVarModels[indx[i]+3,"T"],
    k=mVarModels[indx[i]+3,"k"])$value
  if( i %in% c(1,6:11,13,15,18,20:23) ){  #when (d) provides best fit
    mResidStd <- cbind(mResidStd, coredata(residuals(varMod)) )
    mResidStdNames <- c(mResidStdNames, "4")
  }

  ##(e) log-garch(1,1) w/periodicity, feedback and asymmetry:
  asymext <- mAsymExt[,i] - mean(mAsymExt[,i])
  asymext <- cbind(asymext)
  varMod <- lgarchmc(mMeanModResids[-1,i], arch=1, garch=1,
    xregdet=mDay[,-1], xreg=cbind(mARCH,asymext))
  mVarModels[indx[i]+4,"ARCH"] <- varMod$par["arch1"]
  mVarModels[indx[i]+4,"GARCH"] <- varMod$par["garch1"]
  mVarModels[indx[i]+4,"ASYM"] <- varMod$par["asymext"]
  mVarModels[indx[i]+4,"LogL"] <- as.numeric(logLik(varMod))
  mVarModels[indx[i]+4,"k"] <- length(varMod$par)-1
  mVarModels[indx[i]+4,"T"] <- length(varMod$resids.std)
  mVarModels[indx[i]+4,"BIC"] <- info.criterion(mVarModels[indx[i]+4,"LogL"],
    n=mVarModels[indx[i]+4,"T"],
    k=mVarModels[indx[i]+4,"k"])$value
  if( i %in% c(2:5,12,14,17,24) ){  #when (e) provides best fit (see below)
    mResidStd <- cbind(mResidStd, coredata(residuals(varMod)) )
    mResidStdNames <- c(mResidStdNames, "5")
  }

}
mVarModels


##find the winner models:
indxWinners <- integer(0)
for(i in 1:24){
  whereWinner <- which.min( mVarModels[ indx[i]:c(indx[i]+4) ,"BIC"] )
  indxWinners <- c(indxWinners, indx[i]+whereWinner-1)
}
cbind(indx,indxWinners,indxWinners-indx+1)

##make matrix of winners:
mVarModWinners <- mVarModels[indxWinners,]
mVarModWinners[,"hour"] <- 1:24
mVarModWinners[,"R2"] <- mVarModels[indx,"R2"]
whereAsym0 <- which( is.na(mVarModWinners[,"ASYM"]) )
mVarModWinners[whereAsym0,"ASYM"] <- 0
mVarModWinners

##colnames of winners' residuals:
##(the matrix was constructed during the loop above)
colnames(mResidStd) <- mResidStdNames
mResidStd <- zoo(mResidStd, order.by=index(mMeanModResids)[-1])

##not so fancy plot:
##------------------

tmp <- zoo(mVarModWinners[,c("R2","ARCH","GARCH","ASYM")],
  order.by=1:24)

ylimList <- list()
ylimList[[1]] <- c(0,0.6)
ylimList[[2]] <- c(0,0.2) #earlier: c(-0.1,0.25)
ylimList[[3]] <- c(0.7,1) #earlier: c(0.55,0.95)
ylimList[[4]] <- c(-0.2,0.4) #earlier: c(-0.1,0.6)
plot(tmp, main="", xlab="Hour(m)",
  ylab=c("R-squared","ARCH(1)","GARCH(1)","Leverage"),
  col="blue", lwd=2, ylim=ylimList)

###fancy plot for paper:
###---------------------

##record current graph-values:
def.par <- par(no.readonly=TRUE)

  par( mai=c(0,0.8,0,0) ) #c(bottom, left, top, right)
#  par( mai=c(0,0.6,0,0) ) #c(bottom, left, top, right)

  par(fig=c(0,0.99,0.82,1)) #c(x-start,x-end,y-start,y-end)
  plot(1:24, colMeans(mHourlyPrices), main="", xlab="",
    ylab="Avg. hourly price", cex.lab=0.7,
    type="l", col="blue", ylim=c(34,46), lwd=2,
    yaxt="n", xaxt="n")
  axis(side=2, at=34:46, labels=FALSE)
  axis(side=2, at=c(35,40,45), cex.axis=0.7,
    lwd.ticks=2) #1=below, 2=left, 3=above and 4=right
  abline(h=c(35,40,45), col="grey", lty="dotted")
  abline(v=c(1,5,10,15,20,24), col="grey", lty="dashed")

  par(fig=c(0,0.99,0.64,0.82), new=TRUE)
  plot(1:24, mVarModWinners[,"R2"], main="", xlab="",
    ylab="R-squared", cex.lab=0.7,
    type="l", col="blue", ylim=c(-0.05,0.55), lwd=2,
    yaxt="n", xaxt="n")
  axis(side=2, at=seq(0,0.55,by=0.05), labels=FALSE)
  axis(side=2, at=c(0,0.2,0.4), cex.axis=0.7,
    lwd.ticks=2) #1=below, 2=left, 3=above and 4=right
  abline(h=c(0,0.2,0.4), col="grey", lty="dotted")
  abline(v=c(1,5,10,15,20,24), col="grey", lty="dashed")

  par(fig=c(0,0.99,0.46,0.64), new=TRUE)
  plot(1:24, mVarModWinners[,"ARCH"], main="", xlab="",
    ylab="ARCH(1)", cex.lab=0.7,
    type="l", col="blue", ylim=c(0.04,0.16), lwd=2,
    yaxt="n", xaxt="n")
  axis(side=2, at=seq(0.05,0.15,by=0.01), labels=FALSE)
  axis(side=2, at=c(0.05,0.1,0.15), cex.axis=0.7,
    lwd.ticks=2) #1=below, 2=left, 3=above and 4=right
  abline(h=c(0.05,0.1,0.15), col="grey", lty="dotted")
  abline(v=c(1,5,10,15,20,24), col="grey", lty="dashed")

  par(fig=c(0,0.99,0.28,0.46), new=TRUE)
  plot(1:24, mVarModWinners[,"GARCH"], main="", xlab="",
    ylab="GARCH(1)", cex.lab=0.7,
    type="l", col="blue", ylim=c(0.78,0.93), lwd=2,
    yaxt="n", xaxt="n")
  axis(side=2, at=seq(0.79,0.92,by=0.01), labels=FALSE)
  axis(side=2, at=c(0.8,0.85,0.9), cex.axis=0.7,
    lwd.ticks=2) #1=below, 2=left, 3=above and 4=right
  abline(h=c(0.8,0.85,0.9), col="grey", lty="dotted")
  abline(v=c(1,5,10,15,20,24), col="grey", lty="dashed")

  par(fig=c(0,0.99,0.10,0.28), new=TRUE)
  plot(1:24, mVarModWinners[,"ASYM"], main="",
    ylab="Leverage", cex.lab=0.7,
    type="l", col="blue", ylim=c(-0.18,0.28), lwd=2,
    yaxt="n", xaxt="n")
  axis(side=2, at=seq(-0.15,0.25,by=0.05), labels=FALSE)
  axis(side=2, at=c(-0.1,0.0,0.1,0.2), cex.axis=0.7,
    lwd.ticks=2) #1=below, 2=left, 3=above and 4=right
  abline(h=c(-0.1,0,0.1,0.2), col="grey", lty="dotted")
  abline(v=c(1,5,10,15,20,24), col="grey", lty="dashed")

  axis(side=1, at=1:24, labels=FALSE)
  axis(side=1, at=c(1,5,10,15,20,24), cex.axis=0.7,
    lwd.ticks=2) #1=below, 2=left, 3=above and 4=right
  mtext("Hour(m)", side=1, line=2, cex=0.7)

##return to default margin sizes:
par(def.par, no.readonly=FALSE)

##LaTeX output:
mVarModels <- as.data.frame(mVarModels)
mVarModels[mVarModels[,"log-GARCH"]==1,"log-GARCH"] <- "(a)"
mVarModels[mVarModels[,"log-GARCH"]==2,"log-GARCH"] <- "(b)"
mVarModels[mVarModels[,"log-GARCH"]==3,"log-GARCH"] <- "(c)"
mVarModels[mVarModels[,"log-GARCH"]==4,"log-GARCH"] <- "(d)"
mVarModels[mVarModels[,"log-GARCH"]==5,"log-GARCH"] <- "(e)"
mVarModels
print(xtable(mVarModels, digits=c(0,0,2,0,3,3,3,3,0,4,0)),
  include.rownames=FALSE)


##=========================================================
##5 cDCC MODEL
##=========================================================

## this code is a modified version of code passed on to me
## by Christian Francq. Thanks Christian!

library(compiler)

# Inverse (gnralise) d'une matrice symmetrique
# semi-definie positive
Inv <- function(Sigma, tol=sqrt(.Machine$double.eps)) {
  n<- nrow(Sigma)
  sp<-eigen(Sigma, TRUE)
  sp$vector%*%diag(1/pmax(abs(sp$values),tol))%*%t(sp$vector)
}

### cDCC objective function:
objf.2parRt.DCC <- function(x, eta.star, m, n, r,
  tol=.Machine$double.eps)
{
  aalpha <- x[1]
  bbeta <- x[2]
  if( !all(is.finite(x)) || (aalpha+bbeta)>1-tol ){
    qml <- Inf
  }else{

    omega <- 1-aalpha-bbeta
    Qdiag <- matrix(nrow=n,ncol=m)
    Sstar <- array(dim=c(n,m,m))
    Qdiag[1,] <- 1 #1st observation
    demi.Sstar <- sqrt(Qdiag[1,])*eta.star[1,]
    Sstar[1,,] <- tcrossprod(demi.Sstar)

    tmp.indx <- 2:n
    tmp.fun <- function(i){
     Qdiag[i,] <<- omega + (aalpha*(eta.star[i-1,]^2)+bbeta)*Qdiag[i-1,]
     Sstar[i,,] <<- tcrossprod( sqrt(Qdiag[i,])*eta.star[i,] )
    }
    tmp <- sapply(tmp.indx, tmp.fun)

    Sn <- matrix(0,nrow=m,ncol=m)
    tmp.indx <- (r+1):n
    ##this can be speed-improved??:
    tmp.fun <- function(i){
      Sn <<- Sn+Sstar[i,,]/n
    }
    tmp <- sapply(tmp.indx, tmp.fun)
        Sn.star.moinsdemi <- diag(1/sqrt(diag(Sn)))
    Sn <- Sn.star.moinsdemi %*% Sn %*% Sn.star.moinsdemi

    Qt <- array(dim=c(n,m,m))
    Qt[1,,] <- Sn
    Qt.star <- diag(sqrt(diag(Qt[1,,])))
    Qt.star.inv <- diag(sqrt(1/pmax(Qdiag[1,],tol)))
    Rt <- Qt
    Rt[1,,] <- Qt.star.inv%*%Qt[1,,]%*%Qt.star.inv

    l <- rep(0,n)

    tmp.indx <- 2:n
    tmp.fun <- function(t){
      Qt[t,,] <<- omega*Sn+aalpha*Sstar[t-1,,]+bbeta*Qt[t-1,,]
      Qt.star.inv <<- diag(sqrt(1/pmax(Qdiag[t,],tol)))
      Rt[t,,] <<- Qt.star.inv %*% Qt[t,,] %*% Qt.star.inv
      Rt.inv <- Inv(Rt[t,,])
      l[t] <<- as.numeric( eta.star[t,] %*% Rt.inv %*% eta.star[t,] ) + log(abs(det(Rt[t,,])))
    }
    tmp <- sapply(tmp.indx, tmp.fun)
    assign("aRt", Rt, envir=.GlobalEnv)
    
  ##compute log-likelihood:
  logLikEst <- ( n*log(2*pi)*m + sum(l) )/2
  } #end if(..)else(..)

  return(logLikEst)
} #end function
objf.2parRt.DCC <- cmpfun(objf.2parRt.DCC) #compile

## vech0 function:
vech0 <- function(A)
{
  d <- nrow(A)
  res <- rep(0,(d*(d-1)/2))
  ind <- 0
  for(j in 1:(d-1)){
    for(i in (j+1):d){
      ind <- ind+1
      res[ind] <- A[i,j]
    }
  }
  res
} #end vech0
vech0 <- cmpfun(vech0) #compile

##estimate cDCC:
################

##arguments:
Res <- coredata(mResidStd) #matrix of standardised residuals
m <- NCOL(Res) #
n <- NROW(Res) #
aalpha <- 0.005 #initial value alpha
bbeta <- 0.95 #initial value beta
valinit<-c(aalpha,bbeta)
r <- 10 #burn-in?: discard the first r observations??

#begin:
#{
  res <- nlminb(valinit,objf.2parRt.DCC, lower=c(0.0001,0),
          upper=c(1,1), eta.star=Res, m=m, n=n, r=r)

  res$objective <- -res$objective

  aalpha.est <- res$par[1]
  bbeta.est <- res$par[2]

  omega <- 1-aalpha.est-bbeta.est
  Qdiag <- matrix(nrow=n,ncol=m) #Q*
  Qdiag[1,] <- 1 #initial value
  Sstar <- array(dim=c(n,m,m)) #Q*^{1/2} e'e Q*^{1/2}
  demi.Sstar <- sqrt(Qdiag[1,])*Res[1,] #Q*^{1/2} e'
  Sstar[1,,] <- demi.Sstar%*%t(demi.Sstar) #initial value

  ##recursion of Q*^{1/2} e'e Q*^{1/2}
  for(i in 2:n){
    Qdiag[i,] <- omega+(aalpha.est*(Res[i-1,]^2)+bbeta.est)*Qdiag[i-1,] #Q*
    demi.Sstar <- sqrt(Qdiag[i,])*Res[i,] #Q*^{1/2} e'
    Sstar[i,,] <- demi.Sstar%*%t(demi.Sstar) #Q*^{1/2} e'e Q*^{1/2}
  }

  ##compute estimate of S:
  S.est <- matrix(0,nrow=m,ncol=m)
  for(i in (r+1):n){
     S.est <- S.est+Sstar[i,,]/n
  }
  
  ##the following computes an estimate of the
  ##unconditional correlations (note: estimates of the
  ##unconditional correlations can also be obtained by
  ##cor(mResidStd)):
  S.star.moinsdemi <- diag(1/sqrt(diag(S.est))) #S^{*-1/2}
  S.est <- S.star.moinsdemi %*% S.est %*% S.star.moinsdemi
  #eigen(S.est)

  ##out:
  dccMod <- c(res, list(S=vech0(S.est), aalpha=aalpha.est,
    bbeta=bbeta.est))

#} #end
dccMod

##desc.stats. and plots of unconditional correlations:
summary(dccMod$S)
plot(as.zoo(S.est), main="", xlab="Hour(m)",
  ylab=paste("i=",1:24,sep=""), col="blue", lwd=1,
  ylim=c(-0.05, 1.1))
  
###fancy plot of unconditional correlations for paper:
###---------------------------------------------------

##record current graph-values:
def.par <- par(no.readonly=TRUE)

  par( mai=c(0,0.4,0,0) ) #c(bottom, left, top, right)

  ##left graphs:
  vStep <- 0.07
  vTop <- 0.99
  vBottom <- vTop - vStep
  for(i in 1:12){
    if(i==1){ newGraph <- FALSE }else{ newGraph <- TRUE }
    par(fig=c(0,0.49,vBottom,vTop), new=newGraph) #c(x-start,x-end,y-start,y-end)
    plot(1:24, S.est[,i], main="", xlab="", ylab="",
      type="l",  col="blue", lwd=2, ylim=c(-0.15,1.15),
      yaxt="n", xaxt="n")
    axis(side=2, at=seq(0,1,by=0.2), labels=FALSE, tck=-0.1)
    axis(side=2, at=c(0.0,1.0), cex.axis=0.7,
      lwd.ticks=2, mgp=c(3,0.5,0), tck=-0.1) #1=below, 2=left, 3=above and 4=right
    abline(h=seq(0,1,by=0.2), col="grey", lty="dotted")
    abline(v=c(1,5,10,15,20,24), col="grey", lty="dashed")
    mtext(paste("i=",i,sep=""), side=2, line=1.1, cex=0.8)

    vTop <- vTop - vStep
    vBottom <- vBottom - vStep
  }
  axis(side=1, at=1:24, labels=FALSE, tck=-0.1)
  axis(side=1, at=c(1,5,10,15,20,24), cex.axis=0.7,
    lwd.ticks=2, tck=-0.1, mgp=c(3,0.1,0)) #1=below, 2=left, 3=above and 4=right
  mtext("Hour(m)", side=1, line=1, cex=0.8)

  ##right graphs:
  vTop <- 0.99
  vBottom <- vTop - vStep
  for(i in 13:24){
    par(fig=c(0.51,0.99,vBottom,vTop), new=TRUE) #c(x-start,x-end,y-start,y-end)
    plot(1:24, S.est[,i], main="", xlab="", ylab="",
      type="l",  col="blue", lwd=2, ylim=c(-0.15,1.15),
      yaxt="n", xaxt="n")
    axis(side=2, at=seq(0,1,by=0.2), labels=FALSE, tck=-0.1)
    axis(side=2, at=c(0.0,1.0), cex.axis=0.7,
      lwd.ticks=2, mgp=c(3,0.5,0), tck=-0.1) #1=below, 2=left, 3=above and 4=right
    abline(h=seq(0,1,by=0.2), col="grey", lty="dotted")
    abline(v=c(1,5,10,15,20,24), col="grey", lty="dashed")
    mtext(paste("i=",i,sep=""), side=2, line=1.1, cex=0.8)

    vTop <- vTop - vStep
    vBottom <- vBottom - vStep
  }
  axis(side=1, at=1:24, labels=FALSE, tck=-0.1)
  axis(side=1, at=c(1,5,10,15,20,24), cex.axis=0.7,
    lwd.ticks=2, tck=-0.1, mgp=c(3,0.1,0)) #1=below, 2=left, 3=above and 4=right
  mtext("Hour(m)", side=1, line=1, cex=0.8)

##return to default margin sizes:
par(def.par, no.readonly=FALSE)


##standard deviations of DCCs:
mRtSDs <- matrix(NA,m,m)
for(j in 1:m){
  for(i in 1:m){
    mRtSDs[i,j] <- sd(aRt[,i,j]) #i=row, j=column
  }
}
plot(as.zoo(mRtSDs), main="", xlab="Hour(m)",
  ylab=paste(1:24), col="blue", lwd=1,)

###not-so-fancy plot of DCCs:
###--------------------------

plot(zoo(aRt[,1:24,1], order.by=index(mResidStd)),
  main="", xlab="", ylab=paste(1:24), ylim=c(0.1,1),
  col="blue", lwd=1,)


###fancy plot of DCCs for paper:
###-----------------------------

which1Jan2011 <- which(index(mResidStd)=="2011-01-01")
which1Jan2012 <- which(index(mResidStd)=="2012-01-01")
which1Jan2013 <- which(index(mResidStd)=="2013-01-01")
which1Jan2014 <- which(index(mResidStd)=="2014-01-01")
whereYrs <- c(1, which1Jan2011, which1Jan2012,
  which1Jan2013, which1Jan2014)

##record current graph-values:
def.par <- par(no.readonly=TRUE)

  par( mai=c(0,0.4,0,0) ) #c(bottom, left, top, right)

  ##left graphs:
  vStep <- 0.08
  vTop <- 1
  vBottom <- vTop - vStep
  for(i in 1:12){
    if(i==1){ newGraph <- FALSE }else{ newGraph <- TRUE }
    par(fig=c(0,0.49,vBottom,vTop), new=newGraph) #c(x-start,x-end,y-start,y-end)
    plot(1:length(aRt[,i,1]), aRt[,i,1], main="", xlab="", ylab="",
      type="l",  col="blue", lwd=2, ylim=c(-0.15,1.15),
      yaxt="n", xaxt="n")
    axis(side=2, at=seq(0,1,by=0.2), labels=FALSE, tck=-0.1)
    axis(side=2, at=c(0.0,1.0), cex.axis=0.7,
      lwd.ticks=2, mgp=c(3,0.5,0), tck=-0.1) #1=below, 2=left, 3=above and 4=right
    abline(h=seq(0,1,by=0.2), col="grey", lty="dotted")
    abline(v=whereYrs, col="grey", lty="dashed")
    mtext(paste("m=",i,sep=""), side=2, line=1.1, cex=0.8)

    vTop <- vTop - vStep
    vBottom <- vBottom - vStep
  }
  axis(side=1, at=whereYrs, labels=2010:2014, lwd.ticks=2,
    cex.axis=0.62, tck=-0.1, mgp=c(3,0.1,0)) #1=below, 2=left, 3=above and 4=right

  ##right graphs:
  vTop <- 1
  vBottom <- vTop - vStep
  for(i in 13:24){
    par(fig=c(0.51,0.99,vBottom,vTop), new=TRUE) #c(x-start,x-end,y-start,y-end)
    plot(1:length(aRt[,i,1]), aRt[,i,1], main="", xlab="", ylab="",
      type="l",  col="blue", lwd=2, ylim=c(-0.15,1.15),
      yaxt="n", xaxt="n")
    axis(side=2, at=seq(0,1,by=0.2), labels=FALSE, tck=-0.1)
    axis(side=2, at=c(0.0,1.0), cex.axis=0.7,
      lwd.ticks=2, mgp=c(3,0.5,0), tck=-0.1) #1=below, 2=left, 3=above and 4=right
    abline(h=seq(0,1,by=0.2), col="grey", lty="dotted")
    abline(v=whereYrs, col="grey", lty="dashed")
    mtext(paste("m=",i,sep=""), side=2, line=1.1, cex=0.8)

    vTop <- vTop - vStep
    vBottom <- vBottom - vStep
  }
  axis(side=1, at=whereYrs, labels=2010:2014, lwd.ticks=2,
    cex.axis=0.62, tck=-0.1, mgp=c(3,0.1,0)) #1=below, 2=left, 3=above and 4=right

##return to default margin sizes:
par(def.par, no.readonly=FALSE)


##compute bic for dcc:
dccMod$objective #print logl
info.criterion(dccMod$objective, n=1592, k=2)

###compute bic for unconditional correlations of the dcc:
#LLsum <- 0
#for(i in 1:1592){
#  LLsum <- LLsum + m*log(2*pi) + log(det(S.est)) + rbind(coredata(mResidStd[i,])) %*% solve(S.est) %*% t(coredata(mResidStd[i,]))
#}
#LLsum <- -LLsum/2
#LLsum #print logl
#info.criterion(LLsum, n=1592, k=0)

##compute bic for ccc:
S.estCCC <- cor(mResidStd)
LLsum <- 0
for(i in 1:1592){
  LLsum <- LLsum + m*log(2*pi) + log(det(S.estCCC)) + rbind(coredata(mResidStd[i,])) %*% solve(S.estCCC) %*% t(coredata(mResidStd[i,]))
}
LLsum <- -LLsum/2
LLsum #print logl
info.criterion(LLsum, n=1592, k=0)


##=========================================================
## 6 SUGGESTION BY REVIEWER 2
##=========================================================

##vector of estimation results:
mVarModelsReviewer2 <- matrix(NA, 2, 10)
colnames(mVarModelsReviewer2) <- c("hour", "R2", "log-GARCH",
  "ARCH", "GARCH", "ASYM", "LogL", "k", "BIC", "T")
mVarModelsReviewer2[1:2, "hour"] <- 1
mVarModelsReviewer2[1:2, "R2"] <- mMeanMod[1,"R2"]
mVarModelsReviewer2[1:2, "log-GARCH"] <- 4:5

##(d) log-garch(1,1) w/periodicity and feedback:
mARCH <- cbind(glag(mLogARCH[,24], k=2), glag(mLogARCH[,2]))
varMod <- lgarchmc(mMeanModResids[-c(1:2),1], arch=1, garch=1,
  xregdet=mDay[,-1], xreg=mARCH[-c(1:2),])
mVarModelsReviewer2[1,"ARCH"] <- varMod$par["arch1"]
mVarModelsReviewer2[1,"GARCH"] <- varMod$par["garch1"]
mVarModelsReviewer2[1,"LogL"] <- as.numeric(logLik(varMod))
mVarModelsReviewer2[1,"k"] <- length(varMod$par)-1
mVarModelsReviewer2[1,"T"] <- length(varMod$resids.std)
mVarModelsReviewer2[1,"BIC"] <- info.criterion(mVarModelsReviewer2[1,"LogL"],
  n=mVarModelsReviewer2[1,"T"],
  k=mVarModelsReviewer2[1,"k"])$value

##(e) log-garch(1,1) w/periodicity, feedback and asymmetry:
asymext <- mAsymExt[,1] - mean(mAsymExt[,1])
asymext <- cbind(asymext)
varMod <- lgarchmc(mMeanModResids[-c(1:2),1], arch=1, garch=1,
  xregdet=mDay[,-1], xreg=cbind(mARCH,asymext)[-c(1:2),])
mVarModelsReviewer2[2,"ARCH"] <- varMod$par["arch1"]
mVarModelsReviewer2[2,"GARCH"] <- varMod$par["garch1"]
mVarModelsReviewer2[2,"ASYM"] <- varMod$par["asymext"]
mVarModelsReviewer2[2,"LogL"] <- as.numeric(logLik(varMod))
mVarModelsReviewer2[2,"k"] <- length(varMod$par)-1
mVarModelsReviewer2[2,"T"] <- length(varMod$resids.std)
mVarModelsReviewer2[2,"BIC"] <- info.criterion(mVarModelsReviewer2[2,"LogL"],
  n=mVarModelsReviewer2[2,"T"],
  k=mVarModelsReviewer2[2,"k"])$value

##print results:
print(xtable(mVarModelsReviewer2, digits=c(0,0,2,0,3,3,3,3,0,4,0)),
  include.rownames=FALSE)
